#!/bin/bash

# spdx-license-identifier: gpl-3.0-only
# Copyright (C) 2022 Michał Góral

[[ -n "$LIBGPAK_BUILD_PACKAGES_SH" ]] && return
LIBGPAK_BUILD_PACKAGES_SH=1

source "${LIBRARY}/message.sh"
source "${LIBRARY}/path.sh"
source "${LIBRARY}/exit_codes.sh"

makedeb_local() {
    local distro="$1"
    local pkgdir="$2"
    local pkgname=$(basename -- "$pkgdir")

    (
        export BUILDDIR="${BUILD_DIR}/${distro}/.build/${pkgname}/build"
        export SRCDEST="${BUILD_DIR}/${distro}/.build/${pkgname}/src"
        export SRCPKGDEST="${BUILD_DIR}/${distro}/.build/${pkgname}/srcpkg"
        export PKGDEST="${BUILD_DIR}/${distro}/${pkgname}"

        cd "${pkgdir}"
        makedeb --no-confirm --pass-env
    )
}

makedeb_docker() {
    local distro="$1"
    local pkgdir="$2"
    local pkgname=$(basename -- "$pkgdir")
    local container="gpak:${distro}"
    local outdir="$(make_absolute_path "${BUILD_DIR}/${distro}/${pkgname}")"

    if (( ! FORCE )) && [[ -e "$outdir" ]]; then
        warning2 "Output path %s already exists" "${outdir}"
        return 0
    fi

    mkdir -p "${outdir}"

    docker run \
        --rm \
        -v "${pkgdir}:/script:ro" \
        -v "${outdir}:/out" \
        -e "BUILDDIR=/build/${pkgname}/build" \
        -e "SRCDEST=/build/${pkgname}/sources" \
        -e "SRCPKGDEST=/build/${pkgname}/srcpkg" \
        -e "PKGDEST=/out" \
        --workdir="/script" \
        "${container}" \
        makedeb --sync-deps --no-confirm --pass-env
}

build_packages() {
    local distro="$1" ; shift
    local packages=("$@")
    local called_with_package_list=1
    local ret=0

    if (( ! ${#packages} )); then
        called_with_package_list=0
        shopt -s nullglob
        packages=( "${PACKAGES_DIR}"/* )
        shopt -u nullglob
    fi

    msg "Building packages for %s" "$distro"
    for pkg in "${packages[@]}"; do
        if [[ -e "$pkg/PKGBUILD" ]]; then
            msg2 "Building %s..." "$pkg"
            if (( BUILD_LOCAL )); then
                makedeb_local "$distro" "$pkg"
                [[ $? -ne 0 ]] && ret=$E_BUILD_ERROR
            else
                makedeb_docker "$distro" "$pkg"
                [[ $? -ne 0 ]] && ret=$E_BUILD_ERROR
            fi
        elif (( called_with_package_list )); then
            error "Package doesn't have PKGBUILD file: %s" "$pkg"
            ret=$E_BUILD_ERROR
        fi
    done

    return $ret
}
